/** @page usage Usage of the Library: Include and Link

    Include the headers you need with prefix @c mrw, e.g.:

    @verbatim
    \#include <mrw/stacktrace.hxx>
    @endverbatim

    All classes are in the @c mrw namespace.

    if you want the mrw::StackTrace to evaluate file and line numbers,
    then you need to compile with debug information enabled, thats
    option @c -g.

    Link to the library with option @c -lmrw (or @c -lmrw-mt if your
    program is multi threaded), e.g.:

    @verbatim
    g++ -g -o myprogram myprogram.cxx -lmrw
    @endverbatim

    For the @ref AutoTools "unexpected handler" you need to link to
    @c -lmrwexcstderr or @c -lmrwexclog4cxx (or @c -lmrwexcstderr-mt
    or @c -lmrwexclog4cxx-mt if your program is multi threaded).

    For the @ref AutoFunctionTrace "automated function trace" you need
    to link to @c -lmrwautofunctiontracelog4cxx (or @c
    -lmrwautofunctiontracelog4cxx-mt if your program is multi
    threaded). In addition, you must compile and link your program
    with the GNU compiler specific option @c -finstrument-functions */

/** @page threads Thread Safety

    There is a severe bug in the g++ compiler with version
    >=&nbsp;3.4: The macro @c _REENTRANT is always set and cannot be
    used to check whether we are in a multithreaded compilation or
    not! See http://gcc.gnu.org/bugzilla/show_bug.cgi?id=11953 for
    more details. Workaround: To enable multithreading, you must pass
    the @c -D_MT compiler flag for the GNU gcc verions 3.4 and above!

    To enable thread safety, you must link to @c libmrw-mt instead of
    @c libmrw. That means, your link option is: @c -lmrw-mt

    For GNU gcc verions 3.4 and above, you must use the compile option
    @c -D_MT in addition, because of a severe bug in the compiler.

    Different instances of classes can run in different threads, but
    instances of classes can not be shared between different threads.

    All classes are not thread safe, that means they should either
    always be accessed from the same thread, or you are responsible
    for locking.

    All methods are threads safe, that means, they can be called at
    the same time from different threads. But not on the same object.

    This means: One instance should only be used within one single
    thread. Different instances of the same class can be accessed
    simultaneousely in different threads.

    If you need more thread safety as a feature, please send
    me an email and ask me for it (you find the address in the package
    or on my personal homepage: http://marc.waeckerlin.org).
*/

/** @page libversion Library Versioning

    The library and the package have the same version number. This
    means, if you for example install the package versioned
    mrw-c++-2.6.4.tar.gz, then it builds a shared library of the same
    version, named libmrw.so.2.6.4. The naming is
    version=MAJOR.MINOR.LEAST. What do the numbers mean:

    <dl>
      <dt>MAJOR</dt>
      
      <dd>If the major number is increased, then the backwards
      compatibility was broken. Either there was an incompatible
      change in the implementation, or in the interface, e.g. a change
      in method parameters, or even parts of the library have been
      removed.</dd>

      <dt>MINOR</dt>

      <dd>If the minor number changes, then it remains backward
      compatible. Either there was a minor change in the
      implementation or behavior of some features, a bigger bugfix, or
      new feature were added. A program linked to a previous library
      with smaller minor number still works, but a program linked to
      newer library with a higher minor number may not work with a
      library that has a lower minor number. This is a one way,
      backwards only compatibility.</dd>

      <dt>LEAST</dt>

      <dd>The least number is increased for small changes, bugfixes,
      changes in the documentation and so on. Except the bugs, changes
      in the least number are fully forward and backward
      compatible.</dd>
    </dl>

    This means, programs linked to mrw-c++ version 2.6.4 will work
    with library version 2.6.3, also with version 2.6.5, or with
    version 2.7.1, but it won't work with version 2.5.4 nor with 1.1.3
    nor with 3.1.4.

    You can see the version of the library by issuing the command @c what.
*/

/** @page license License
    @verbinclude COPYING */

/** @page install Installation
    @verbinclude INSTALL */

/** @page news News
    @verbinclude NEWS */

/** @page changes Change Log
    @verbinclude ChangeLog */

/** @brief the namespace for this library

    All classes and all functions (except global operators) are placed
    in this namespace.

    For an overview, better see the <a
    href="modules.html">"Modules"</a> page.
*/
namespace mrw {
}
